// java

package edu.washington.cac.calendar;

import java.sql.Time;
import java.util.Calendar;
import java.util.Vector;

import javax.servlet.http.HttpServletRequest;

import edu.washington.cac.calendar.data.Cacheable;
import edu.washington.cac.calendar.data.DataChecker;
import edu.washington.cac.calendar.data.CaldataException;
import edu.washington.cac.myuw.servlet.UWParameter;
import edu.washington.cac.myuw.servlet.ParameterException;

import uwcal.out.DisplayText;
import uwcal.out.forms.FormsDisplayText;

/** 
  UWParameter class with Calendar-specific methods
  @author Greg Barnes
  @version 1.2
 */
public class CalParameter extends UWParameter 
{
  /** Create a <code>CalParameter</code> from an HTTP request
    @param req The <code>javax.servlet.http.HttpServletRequest</code>
       for a servlet.
   */
  public CalParameter(HttpServletRequest req) {
    super(req);
  }

  /** Parameter name:  The main function the servlet should perform */
  public static final String FUNCTION = "function";

  /** Parameter name:  The secondary function the servlet should perform */
  public static final String FUNCTION2 = "function2";

  /** Parameter name:  The date in <code>YYYYMMDD</code> form */
  public static final String DATE = "date";

  /** Parameter name:  The start date of an event in <code>YYYYMMDD</code> 
      form 
   */
  public static final String START_DATE = "start_date";

  /** Parameter name:  The end date of an event in <code>YYYYMMDD</code> form */
  public static final String END_DATE = "end_date";

  /** Parameter name:  The unique id of an event */
  public static final String EVENT_ID = "eventid";

  /** the parameter name of the session identifier */
  public static final String IDENTIFIER_NAME = "identifier";

  /** Parameter name:  A short description of an event */
  public static final String SHORT_DESCRIPTION = "shortdesc";

  /** Parameter name:  A long description of an event */
  public static final String LONG_DESCRIPTION = "longdesc";

  /** Parameter name:  A link for an event, location, or sponsor */
  public static final String LINK = "link_string";

  /** Parameter name:  Whether an event, location, sponsor, or keyword 
              is public
    */
  public static final String PUBLIC = "public";

  /** Parameter name:  The cost of an event */
  public static final String COST = "cost";

  /** Parameter name:  The hour field of an event's starting time */
  public static final String START_HOUR = "shour";

  /** Parameter name:  The minute field of an event's starting time */
  public static final String START_MINUTE = "smin";

  /** Parameter name:  The am/pm field of an event's starting time */
  public static final String START_AMPM = "sampm";

  /** Parameter name:  The hour field of an event's ending time */
  public static final String END_HOUR = "ehour";

  /** Parameter name:  The minute field of an event's ending time */
  public static final String END_MINUTE = "emin";

  /** Parameter name:  The am/pm field of an event's ending time */
  public static final String END_AMPM = "eampm";

  /** Parameter name:  The month field of an event's starting date */
  public static final String START_MONTH = "smonth";

  /** Parameter name:  The day field of an event's starting date */
  public static final String START_DAY = "sday";

  /** Parameter name:  The year field of an event's starting date */
  public static final String START_YEAR = "syear";

  /** Parameter name:  The month field of an event's ending date */
  public static final String END_MONTH = "emonth";

  /** Parameter name:  The day field of an event's ending date */
  public static final String END_DAY = "eday";

  /** Parameter name:  The year field of an event's ending date */
  public static final String END_YEAR = "eyear";

  /** Parameter name:  The unique id of a keyword */
  public static final String KEYWORD_ID = "keywordid";

  /** Parameter name:  The name of a keyword */
  public static final String WORD = "word";

  /** Parameter name:  The unique id of a location */
  public static final String LOCATION_ID = "locationid";

  /** Parameter name:  The main address of a location */
  public static final String ADDRESS = "address";

  /** Parameter name:  The secondary address of a location */
  public static final String SUBADDRESS = "subaddress";

  /** Parameter name:  The unique id of a sponsor */
  public static final String SPONSOR_ID = "sponsorid";

  /** Parameter name:  The name of a sponsor */
  public static final String SPONSOR_NAME = "name";

  /** Parameter name:  The phone of a sponsor */
  public static final String PHONE = "phone";

  /** Parameter name:  The email address of a sponsor */
  public static final String EMAIL = "email";

  /** Parameter name: A submit button (in case we have more than one that
    do different things
   */
  public static final String SUBMIT = "submit";

  /** Parameter name: A string to filter public events */
  public static final String SEARCH_STRING = "searchString";

  /** Parameter name: the id of a public events calendar */
  public static final String CALENDAR = "calendar";

  /** Parameter name: should the week/month view be shown as a grid or a list */
  public static final String SHOW_GRID = "showgrid";

  // parameters associated with 'add schedule'
  /** Parameter name: the type of a schedule to be added */
  public static final String SCHEDULE = "schedule";

  /** Parameter name: the quarter */
  public static final String QUARTER = "quarter";

  /** Parameter name: list of schedule ids */
  public static final String SCHEDULE_IDS = "ids";

  // common parameter values:

  //   operations
  /** Parameter value prefix for enter function */
  private static final String ENTER = "enter";

  /** Parameter value prefix for edit function */
  private static final String EDIT = "edit";

  /** Parameter value prefix for add function */
  public static final String ADD = "add";

  /** Parameter value prefix for delete function */
  public static final String DELETE = "del";

  /** Parameter value prefix for delete function for recurring events */
  public static final String RDELETE = "rdel";

  /** Parameter value prefix for replace function */
  public static final String REPLACE = "replace";

  private static String enter(String entity) 
  {
    return ENTER + entity;
  }

  private static String edit(String entity) 
  {
    return EDIT + entity;
  }

  private static String add(String entity) 
  {
    return ADD + entity;
  }

  /**
    Get the name of a delete operation
    @param entity Type of entity to delete
    @return the name of a delete operation
   */
  private static String delete(String entity) 
  {
    return DELETE + entity;
  }

  /**
    Get the name of a delete operation on a recurring entity
    @param entity Type of entity to delete
    @return the name of a delete operation on a recurring entity
   */
  private static String deleteRecur(String entity) 
  {
    return RDELETE + entity;
  }

  private static String replace(String entity) {
    return REPLACE + entity;
  }

  //   entity names
  /** Parameter value suffix for event */
  public static final String EVENT = "entry";
  /** Parameter value suffix for fullevent */
  public static final String FULLEVENT = "full";
  /** Parameter value suffix for keyword */
  public static final String KEYWORD = "kw";
  /** Parameter value suffix for sponsor */
  public static final String SPONSOR = "sponsor";
  /** Parameter value suffix for location */
  public static final String LOCATION = "location";
  /** Parameter value suffix for eventref */
  public static final String EVENTREF = EVENT + "ref";

  /** Parameter values: list of UWCal functions that only display a page */
  public static final String[] DISPLAY_FUNCTIONS = {
    "dayview", "weekview", "monthview", "yearview", "eventview", "go",
    "allcalendars",
    edit(EVENT), 
    enter(FULLEVENT), edit(FULLEVENT), 
    enter(KEYWORD), edit(KEYWORD), 
    enter(SPONSOR), edit(SPONSOR), 
    enter(LOCATION), edit(LOCATION)
  };

  /** Parameter values: list of UWCal functions that perform add or replace 
      operations 
   */
  public static final String[] OPERATIONS = {
    add(EVENT), replace(EVENT),
    add(KEYWORD), replace(KEYWORD),
    add(SPONSOR), replace(SPONSOR),
    add(LOCATION), replace(LOCATION)
  };

  /** Parameter values: list of UWCal functions that perform delete 
      operations 
   */
  public static final String[] DELETE_OPERATIONS = {
    delete(EVENT), delete(KEYWORD), delete(SPONSOR), 
    delete(LOCATION)
  };

  public static final String[] REF_OPERATIONS = {
    add(EVENTREF), delete(EVENTREF), deleteRecur(EVENTREF),
    add(CALENDAR), delete(CALENDAR)
  };

  /** Parameter value: list of UWCal functions that change internal variables,
      but not the database
   */
  public static final String[] FILTER_OPERATIONS = {
    "selectfilter"
  };

  //   more convenient names for the items in the above arrays

  /** Parameter value: function to show a day */
  public static final String DAYVIEW_FUNC = DISPLAY_FUNCTIONS[0];
  /** Parameter value: function to show a week */
  public static final String WEEKVIEW_FUNC = DISPLAY_FUNCTIONS[1];
  /** Parameter value: function to show a month */
  public static final String MONTHVIEW_FUNC = DISPLAY_FUNCTIONS[2];
  /** Parameter value: function to show a year */
  public static final String YEARVIEW_FUNC = DISPLAY_FUNCTIONS[3];
  /** Parameter value: function to show an event */
  public static final String EVENTVIEW_FUNC = DISPLAY_FUNCTIONS[4];
  /** Parameter value: function to go to a particular day */
  public static final String GO_FUNC = DISPLAY_FUNCTIONS[5];
  /** Parameter value: function to see the list of all calendars */
  public static final String ALLCALENDARS_FUNC = DISPLAY_FUNCTIONS[6];
  /** Parameter value: function to edit an event */
  public static final String EDITEVENT_FUNC = DISPLAY_FUNCTIONS[7];
  /** Parameter value: function to enter a full event */
  public static final String ENTERFULL_FUNC = DISPLAY_FUNCTIONS[8];
  /** Parameter value: function to edit a fullevent */
  public static final String EDITFULL_FUNC = DISPLAY_FUNCTIONS[9];
  /** Parameter value: function to enter a keyword */
  public static final String ENTERKEYWORD_FUNC = DISPLAY_FUNCTIONS[10];
  /** Parameter value: function to edit a keyword */
  public static final String EDITKEYWORD_FUNC = DISPLAY_FUNCTIONS[11];
  /** Parameter value: function to enter a sponsor */
  public static final String ENTERSPONSOR_FUNC = DISPLAY_FUNCTIONS[12];
  /** Parameter value: function to edit a sponsor */
  public static final String EDITSPONSOR_FUNC = DISPLAY_FUNCTIONS[13];
  /** Parameter value: function to enter a location */
  public static final String ENTERLOCATION_FUNC = DISPLAY_FUNCTIONS[14];
  /** Parameter value: function to edit a location */
  public static final String EDITLOCATION_FUNC = DISPLAY_FUNCTIONS[15];
  
  /** Parameter value: function to add an event */
  public static final String ADDEVENT_FUNC = OPERATIONS[0];
  /** Parameter value: function to replace an event */
  public static final String REPLACEEVENT_FUNC = OPERATIONS[1];
  /** Parameter value: function to add a keyword */
  public static final String ADDKEYWORD_FUNC = OPERATIONS[2];
  /** Parameter value: function to replace a keyword */
  public static final String REPLACEKEYWORD_FUNC = OPERATIONS[3];
  /** Parameter value: function to add a sponsor */
  public static final String ADDSPONSOR_FUNC = OPERATIONS[4];
  /** Parameter value: function to replace a sponsor */
  public static final String REPLACESPONSOR_FUNC = OPERATIONS[5];
  /** Parameter value: function to add a location */
  public static final String ADDLOCATION_FUNC = OPERATIONS[6];
  /** Parameter value: function to replace a location */
  public static final String REPLACELOCATION_FUNC = OPERATIONS[7];

  /** Parameter value: function to delete an event */
  public static final String DELETEEVENT_FUNC = DELETE_OPERATIONS[0];
  /** Parameter value: function to delete a keyword */
  public static final String DELETEKEYWORD_FUNC = DELETE_OPERATIONS[1];
  /** Parameter value: function to delete a sponsor */
  public static final String DELETESPONSOR_FUNC = DELETE_OPERATIONS[2];
  /** Parameter value: function to delete a location */
  public static final String DELETELOCATION_FUNC = DELETE_OPERATIONS[3];

  /** Parameter value: function to add an eventref */
  public static final String ADDEVENTREF_FUNC = REF_OPERATIONS[0];
  /** Parameter value: function to remove a reference to an event*/
  public static final String REMOVEEVENTREF_FUNC = REF_OPERATIONS[1];
  /** Parameter value: function to remove a reference to a recurring event*/
  public static final String REMOVEALLEVENTREF_FUNC = REF_OPERATIONS[2];
  /** Parameter value: function to subscribe to a calendar */
  public static final String SUBSCRIBE_FUNC = REF_OPERATIONS[3];
  /** Parameter value: function to unsubscribe to a calendar */
  public static final String UNSUBSCRIBE_FUNC = REF_OPERATIONS[4];

  /** Parameter value: function to change the filter for a guest view */
  public static final String SELECTFILTER_FUNC = FILTER_OPERATIONS[0];

  //   other common parameter values
  /** Parameter value:  Null */
  public static final String NULL = "NULL";

  /** Parameter value:  What gets entered by default into a link 
      textbox */
  public static final String DEFAULT_LINK = "http://";

  /** Parameter value: student schedule type */
  public static final char STUDENT_SCHEDULE = 'S';

  /** Parameter value: teaching schedule type */
  public static final char TEACHING_SCHEDULE = 'T';

  /** The prefix of the 'select all' calendar in the Select Filters form */
  public static final String SELECT_ALL = "all";

  /** The prefix of the default calendar in the Select Filters form */
  public static final String DEFAULT_FILTER = "def";

  /** The prefix of a special filter in the Select Filters form */
  public static final String SPECIAL_FILTER = "Special";

  /** 
    Get the value of the <code>DATE</code> parameter

    @return the value of the <code>DATE</code> parameter
    @exception ParameterException if that parameter was 
       not set
    @exception CaldataException if the parameter does not 
       represent a valid date
   */
  public String getDateString() throws ParameterException, CaldataException
  {
    return new DataChecker().checkDateString(getPValue(DATE, false));
  }
    
  /**
    Return the start year
    @return The start year
    @exception ParameterException if the parameter doesn't exist
   */
  public String getStartYear() throws ParameterException
  {
    return getPValue(START_YEAR);
  }
  
  /**
    Return the start month
    @return The start month
    @exception ParameterException if the parameter doesn't exist
   */
  public String getStartMonth() throws ParameterException
  {
    return getPValue(START_MONTH);
  }
  
  /**
    Return the start day
    @return The start day
    @exception ParameterException if the parameter doesn't exist
   */
  public String getStartDay() throws ParameterException
  {
    return getPValue(START_DAY);
  }

  /**
    Get the start date, as a string
    @return The start date, as a <code>String</code>
    @exception ParameterException if a parameter does not exist
    @exception CaldataException if the parameters have bad data
   */
  private String getStartdateString()
      throws ParameterException, CaldataException
  {
    return new DataChecker()
           .checkDateString(getStartYear() + getStartMonth() + getStartDay());
  }

  /**
    Return the end year
    @return The end year
    @exception ParameterException if the parameter doesn't exist
   */
  public String getEndYear() throws ParameterException
  {
    return getPValue(END_YEAR).equals(FormsDisplayText.SAME) 
           ? getPValue(START_YEAR) : getPValue(END_YEAR);
  }
  
  /**
    Return the end month
    @return The end month
    @exception ParameterException if the parameter doesn't exist
   */
  public String getEndMonth() throws ParameterException
  {
    return getPValue(END_MONTH).equals(FormsDisplayText.SAME) 
           ? getPValue(START_MONTH) : getPValue(END_MONTH);
  }
  
  /**
    Return the end day
    @return The end day
    @exception ParameterException if the parameter doesn't exist
   */
  public String getEndDay() throws ParameterException
  {
    return getPValue(END_DAY).equals(FormsDisplayText.SAME) 
           ? getPValue(START_DAY) : getPValue(END_DAY);
  }
  
  /**
    Get the end date, as a string
    @return The end date, as a <code>String</code>
    @exception ParameterException if a parameter does not exist
    @exception CaldataException if the parameters have bad data
   */
  private String getEnddateString() throws ParameterException, CaldataException
  {
    return new DataChecker()
           .checkDateString(getEndYear() + getEndMonth() + getEndDay());
  }
    
  /** Get the value of a string parameter

   @param name The name of the parameter to get
   @return the value of a named parameter, or null if that 
     parameter was not set
   */
  protected String getString(String name) 
  {
    try {
      return DataChecker.normalize(getPValue(name));
    } catch (ParameterException e) {
      return null;
    }
  }

  /** The default page to display if none is specified or something goes
      wrong */
  public static final String DEFAULT_VIEW =
      CalProperties.defaultIsGuest() ? WEEKVIEW_FUNC : DAYVIEW_FUNC;

  /** 
    Get the value of the <code>FUNCTION</code> parameter

    @return the value of the <code>FUNCTION</code> parameter, or
    the name of the default function if that parameter was not set
   */
  public String getFunction() 
  {
    if (getString(FUNCTION) == null) {
        return DEFAULT_VIEW;
    } else {
        return getString(FUNCTION);
    }
  }

  /** 
    Get the value of the <code>FUNCTION2</code> parameter

    @return the value of the <code>FUNCTION2</code> parameter, or
    the name of the default function if that parameter was not set
    or was set to an invalid value
   */
  public String getFunction2() 
  {
    if (getString(FUNCTION2) == null || 
        !inArray(getString(FUNCTION2), DISPLAY_FUNCTIONS))
    {
        return DAYVIEW_FUNC;
    } else {
        return getString(FUNCTION2);
    }
  }

  /** @return Is the String in the array?
    @param s String to test
    @param a Array to test
   */
  public boolean inArray(String s, String[] a) 
  {
    for (int i = 0; i < a.length; i++) {
      if (a[i].equals(s)) {
        return true;
      }
    }

    return false;
  }
     
  /** 
     Is the <code>FUNCTION</code> parameter one that can only
     be performed by a user?
     @return Is the <code>FUNCTION</code> parameter one that can only
     be performed by a user?
   */
  public boolean isUserFunction()
  {
    if (inArray(getFunction(), DISPLAY_FUNCTIONS)) {
      return false;
    } else if (inArray(getFunction(), FILTER_OPERATIONS)) {
      return false;
    } else if (inArray(getFunction(), OPERATIONS)) {
      return true;
    } else if (inArray(getFunction(), REF_OPERATIONS)) {
      return true;
    } else {
      return inArray(getFunction(), DELETE_OPERATIONS);
    }
  }

    /** 
        Get the value of the <code>LONG_DESCRIPTION</code> parameter

        @return the value of the <code>LONG_DESCRIPTION</code> parameter
      */
  public String getLongdesc()
  {
      return getString(LONG_DESCRIPTION);
  }

    /** 
        Get the value of the <code>SHORT_DESCRIPTION</code> parameter
        @return the value of the <code>SHORT_DESCRIPTION</code> parameter
      */
  public String getShortdesc()
  {
      return getString(SHORT_DESCRIPTION);
  }

    /** 
        Get the value of the <code>COST</code> parameter
        @return the value of the <code>COST</code> parameter
      */
  public String getCost()
  {
      return getString(COST);
  }

  /** 
      Get the value of the parameter that holds a URL
      @param name Name of the parameter
      @return the URL
    */
  protected String getUrl(String name)
  {
    if (getString(name) == null || getString(name).equals(DEFAULT_LINK) || 
      getString(name).equals("")) 
    {
        return null;
    } else {
        return getString(name);
    }
  }

  /** 
      Get the value of the <code>LINK</code> parameter
      @return the value of the <code>LINK</code> parameter
    */
  public String getLink()
  {
    return getUrl(LINK);
  }

    /** 
        Get the value of the <code>WORD</code> parameter
        @return the value of the <code>WORD</code> parameter
      */
  public String getWord()
  {
      return getString(WORD);
  }

    /** 
        Get the value of the <code>SPONSOR_NAME</code> parameter
        @return the value of the <code>SPONSOR_NAME</code> parameter
      */
  public String getSponsorName() {
      return getString(SPONSOR_NAME);
  }
      
    /** 
        Get the value of the <code>PHONE</code> parameter
        @return the value of the <code>PHONE</code> parameter
      */
  public String getPhone() {
      return getString(PHONE);
  }
      
    /** 
        Get the value of the <code>EMAIL</code> parameter
        @return the value of the <code>EMAIL</code> parameter
      */
  public String getEmail() {
      return getString(EMAIL);
  }
      
    /** 
        Get the value of the <code>ADDRESS</code> parameter
        @return the value of the <code>ADDRESS</code> parameter
      */
  public String getAddress() {
      return getString(ADDRESS);
  }
      
  /** 
    Get the value of the <code>SUBADDRESS</code> parameter
    @return the value of the <code>SUBADDRESS</code> parameter
   */
  public String getSubaddress() {
    return getString(SUBADDRESS);
  }
        
  /** 
    Get the value of the <code>SUBMIT</code> parameter
    @return the value of the <code>SUBMIT</code> parameter
   */
  public String getSubmit() 
  {
    return getString(SUBMIT);
  }
        
  /** 
    Get the value of the <code>SEARCH_STRING</code> parameter
    @return the value of the <code>SEARCH_STRING</code> parameter
   */
  public String getSearchString() 
  {
    return getString(SEARCH_STRING);
  }

  /** 
    Get the value of the <code>CALENDAR</code> parameter
    @return the value of the <code>CALENDAR</code> parameter
   */
  public int getCalendar() 
  {
    return Integer.parseInt(getString(CALENDAR));
  }

  /** 
    Get all the values of the <code>CALENDAR</code> parameter
    @return all the values of the <code>CALENDAR</code> parameter
   */
  public int[] getCalendars() 
  {
    return getInts(CALENDAR);
  }

  /** 
    Get the value of the <code>SCHEDULE</code> parameter
    @return the value of the <code>SCHEDULE</code> parameter
   */
  public char getSchedule() 
  {
    if (getString(SCHEDULE) != null && 
        getString(SCHEDULE).equalsIgnoreCase(TEACHING_SCHEDULE + "")) 
    {
      return TEACHING_SCHEDULE;
    } else {
      return STUDENT_SCHEDULE;
    }
  }

  /** 
    Get the value of the <code>QUARTER</code> parameter, as an integer.
    @return the value of the <code>QUARTER</code> parameter
   */
  //Integer conversion should thwart any malicious parameter spoofing
  public int getQuarter() 
  {
    return Integer.parseInt(getString(QUARTER));
  }

  /** A prefix to add to all schedule ids */
  private static final String SCHEDULE_PREFIX = "SLN";

  /**
    Get all the ids of the schedule events 
    @return all the ids of the schedule events 
   */
  public Vector getScheduleIds()
  {
    Vector v = new Vector();
    String rawIds = getString(SCHEDULE_IDS);

    if (rawIds == null) {
      return v;
    }

    while (rawIds.indexOf(',') != -1) {
      v.add(SCHEDULE_PREFIX + rawIds.substring(0, rawIds.indexOf(',')));
      rawIds = rawIds.substring(rawIds.indexOf(',') + 1);
    }

    // get around unnecessary trailing comma
    if (rawIds.trim().length() > 0) {
      v.add(SCHEDULE_PREFIX + rawIds);
    }

    return v;
  }

  /**
    Get the values of a parameter that returns a set of Strings
    @param name Name of the parameter
    @return the values of a parameter that returns a set of Strings
   */
  public String[] getStrings(String name) 
  {
    String[] values;

    try {
      values = getPValues(name, UWParameter.ANY, true, false);

      if (values == null) {
        return null;
      }

      for (int i = 0; i < values.length; i++) {
        values[i] = DataChecker.normalize(values[i]);
      }
    } catch (ParameterException e) {
      /* If anything's wrong, we return null.  Don't mess around with goofy
         parameters, as it's probably a spoof attempt
       */
      return null;
    }

    return values;
  }

  /**
    Get the values of a parameter that returns a set of integers
    @param name Name of the parameter
    @return the values of a parameter that returns a set of integers
   */
  public int[] getInts(String name) 
  {
    try {
      String[] sValues = getPValues(name, UWParameter.ANY, true, false);

      if (sValues == null) {
        return null;
      }

      int[] values = new int[sValues.length];

      for (int i = 0; i < values.length; i++) {
        values[i] = Integer.parseInt(sValues[i]);
      }

      return values;
    } catch (ParameterException e) {
      /* If anything's wrong, we return null.  Don't mess around with goofy
         parameters, as it's probably a spoof attempt
       */
      return null;
    }
  }

  /** 
    Test if a parameter has a value,
    @param name The name of the parameter
    @return true if the parameter has a value, false otherwise
   */
  protected boolean getCheckValue(String name) 
  {
    try {
      return getPValue(name, false) == null ? false : true;
    } catch (ParameterException e) {
      return false;
    }
  }

  /** 
    Does the <code>PUBLIC</code> parameter have a value?
    @return Does the <code>PUBLIC</code> parameter have a value?
   */
  public boolean getPublic() 
  {
    return getCheckValue(PUBLIC);
  }

  /** 
    Does the <code>SHOW_GRID</code> parameter have a value?
    @return Does the <code>SHOW_GRID</code> parameter have a value?
   */
  public boolean getShowGrid() 
  {
    return getCheckValue(SHOW_GRID);
  }

  /**
    Get an id
    @param name Name of the id
    @return the id, or <code>Cacheable.NO_ID</code> if there was a problem
   */
  private int getID(String name) 
  {
    try {
      return Integer.parseInt(getPValue(name));
    } catch (ParameterException e) {
      return Cacheable.NO_ID;
    }
  }

  /** 
    Get the value of the <code>EVENT_ID</code> parameter

    @return the <code>int</code> value of the <code>EVENT_ID</code>
      parameter, or <code>uwcal.data.Cacheable.NO_ID</code> if
      the parameter was not set
   */
  public int getEventID()
  {
    return getID(EVENT_ID);
  }

  /** 
    Get the value of the <code>KEYWORD_ID</code> parameter

    @return the <code>int</code> value of the <code>KEYWORD_ID</code>
      parameter, or <code>uwcal.data.Cacheable.NO_ID</code> if
      the parameter was not set
   */
  public int getKeywordID()
  {
    return getID(KEYWORD_ID);
  }

  /** 
    Get the value of the <code>LOCATION_ID</code> parameter

    @return the <code>int</code> value of the <code>LOCATION_ID</code>
      parameter, or <code>uwcal.data.Cacheable.NO_ID</code> if
      the parameter was not set
   */
  public int getLocationID()
  {
    return getID(LOCATION_ID);
  }

  /** 
    Get the value of the <code>SPONSOR_ID</code> parameter
    @return the <code>int</code> value of the <code>SPONSOR_ID</code>
      parameter, or <code>uwcal.data.Cacheable.NO_ID</code> if
      the parameter was not set
   */
  public int getSponsorID()
  {
    return getID(SPONSOR_ID);
  }

  /** 
    Get the value of the <code>END_DATE</code> parameter
    @return the value of the <code>END_DATE</code> parameter
      as a <code>java.sql.Date</code>
    @exception CaldataException if the String was not in a valid form
   */
  public java.sql.Date getEnddate() throws CaldataException
  {
    try {
      return new java.sql.Date(
          new MyCalendar(getEnddateString()).getTime().getTime());
    } catch (ParameterException e) {
      return getStartdate();
    }
  }

  /** 
    Get the value of the <code>START_DATE</code> parameter

    @return the value of the <code>START_DATE</code> parameter
      as a <code>java.sql.Date</code>
    @exception CaldataException if the String was not in a valid form
   */
  public java.sql.Date getStartdate() throws CaldataException
  {
    try {
      return new java.sql.Date(
          new MyCalendar(getStartdateString()).getTime().getTime());
    } catch (ParameterException e) {
      return null;
    }
  }

  private int hourAs12(String hour) 
  {
    return Integer.parseInt(hour);
  }

  private int hourAs24(int hour, String ampm) 
  {
    if (ampm.equals(FormsDisplayText.PM)) {
      return hour == 12 ? hour : hour + 12;
    } else {
      return hour == 12 ? 0 : hour;
    }
  }
      
  private int getMinute(String minute) 
  {
    return Integer.parseInt(minute);
  }

  private Time constructTime(String hour, String minute, String ampm)
      throws CaldataException 
  {
    if (hour.equals(DisplayText.NO_VALUE)) {
      return null;
    }

    /* First check that hour and minute are in suitable format.
       (ampm can't really screw us up, so let it go)
     */
    new DataChecker().checkTime(hourAs12(hour), getMinute(minute));

    Calendar c = Calendar.getInstance();
    /* java.sql.Time's are supposed to have the 'date' components
       set to January 1, 1970 */
    c.set(1970, Calendar.JANUARY, 1, hourAs24(hourAs12(hour), ampm),
      getMinute(minute), 0);
    c.set(Calendar.MILLISECOND, 0);
    return new Time(c.getTime().getTime());
  }

  /** 
    Get an event's ending time 

    @return an event's ending time as indicated by the values of the
      <code>END_HOUR</code>, <code>END_MINUTE</code>, and
      <code>END_AMPM</code> parameters, as a <code>java.sql.Time</code>
    @exception CaldataException if the hour or minute are not valid
   */
  public Time getEndtime() throws CaldataException
  {
    try {
      return constructTime(getPValue(END_HOUR), getPValue(END_MINUTE),
                           getPValue(END_AMPM));
    } catch (ParameterException e) {
      return null;
    }
  }

  /** 
    Get an event's starting time 
    @return an event's starting time as indicated by the values of the
      <code>START_HOUR</code>, <code>START_MINUTE</code>, and
      <code>START_AMPM</code> parameters, as a <code>java.sql.Time</code>
    @exception CaldataException if the hour or minute are not valid
   */
  public Time getStarttime() throws CaldataException
  {
    try {
      return constructTime(getPValue(START_HOUR), getPValue(START_MINUTE),
                           getPValue(START_AMPM));
    } catch (ParameterException e) {
      return null;
    }
  }

  /** Is the <code>FUNCTION</code> parameter addeventref?
      @return is the <code>FUNCTION</code> parameter addeventref
   */
  public boolean isAddEventRefFunction()
  {
    return getFunction().equals(ADDEVENTREF_FUNC);
  }

  /** 
    Get the session identifier value
    @return the session identifier value (integer)
    @exception ParameterException if that parameter was not set
   */
  public int getIdentifier () throws ParameterException
  {
    return Integer.parseInt(getPValue(IDENTIFIER_NAME));
  }
}
