// java

package edu.washington.cac.calendar.data;

import java.sql.SQLException;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Vector;

import edu.washington.cac.calendar.db.Caldata;
import edu.washington.cac.myuw.servlet.session.UWSession;

/**
  A set of cached items
  @author Greg Barnes
  @version 1.1
 */
public class Cache implements CacheI
{
  protected HashMap hash = new HashMap();

  /** The user who owns the cache */
  protected User user;

  /** An object to do db operations */
  private Caldata caldata;

  /** Create a cache for a specific user
    @param user The user
    @param caldata An object that can do db operations
   */
  public Cache(User user, Caldata caldata) {
    this.user = user;
    this.caldata = caldata;
  }

  /** Create a cache for the guest
    @param caldata An object that can do db operations
   */
  public Cache(Caldata caldata) {
    this(new User(), caldata);
  }

  /** Create a cache for a specific user
    @param user The user
   */
  public Cache(User user) {
    this(user, null);
  }

  /** Create a cache for the guest
   */
  public Cache() {
    this(null, null);
  }

  /** Get the Caldata object.  Will create one if one has not been set. */
  protected Caldata getCaldata() {
    if (this.caldata == null) {
      this.caldata = new Caldata();
    }

    return this.caldata;
  }

  /**
    Get the number of items in the Cache
    @return The number of items in the Cache
   */
  public int size() 
  {
    return hash.size();
  }

  /**
    Get all the items in the cache, as an enumeration
    @return All the items in the cache
   */
  public Iterator elements() 
  {
    return hash.values().iterator();
  }

  /**
    Get all the items in the cache, as a Collection
    @return All the items in the cache
   */
  public Collection values() 
  {
    return hash.values();
  }

  /**
    Turn an integer id into an object suitable to be a key
    @param id The integer id
    @return the id, as an object 
   */
  protected Integer key(int id) 
  {
    return new Integer(id);
  }

  /**
    Get the key for a Cacheable object
    @param c The Cacheable
    @return the key for the Cacheable object
   */
  protected Integer key(Cacheable c) 
  {
    return key(c.getId());
  }

  /**
    Add an item to the cache
    @param c item to add
   */
  protected void localAdd(Cacheable c) 
  {
    hash.put(key(c), c);
    invalidateSort();
  }

  /**
    Add an item to the cache, and save it to the database
    @param c Item to add
    @exception SQLException If there's a db problem
   */
  public void add(Cacheable c) throws SQLException
  {
    add(c, true);
  }

  /**
    Add an item to the cache
    @param c Item to add
    @param saveToDB Save to database as well?
    @exception SQLException If there's a db problem
   */
  public void add(Cacheable c, boolean saveToDB) 
    throws SQLException
  {
    if (saveToDB) {
      c.add(this.user);
    } 

    if (!hash.containsKey(key(c))) {
      localAdd(c);
    }
  }

  /**
    Add a collection of <code>Cacheable</code> items to the cache
    @param c Collection of items
    @param saveToDB Save to database as well?
    @exception SQLException If there's a db problem
   */
  public void addAll(Collection c, boolean saveToDB) 
      throws SQLException
  {
    Iterator i = c.iterator();
    
    while (i.hasNext()) {
      add((Cacheable) i.next(), saveToDB);
    }
  }

  /**
    Delete an item from this cache, but not from the database
    @param c Item to delete
    @exception NoSuchItemException if there is no current item with
      the same id as c
    @exception SQLException if there is a problem with the database
   */
  protected void localDelete(Cacheable c) 
      throws SQLException, NoSuchItemException
  {
    if (hash.containsKey(key(c))) {
      hash.remove(key(c));
    } else {
      throw new NoSuchItemException("localDelete: couldn't find item " + 
                                    key(c));
    }

    invalidateSort();
  }

  /**
    delete an item from the Cache 

    @param c the item to delete
    @exception SQLException if there is a problem with the database
    @exception ItemException if there's a problem with c 
   */
  public void delete(Cacheable c) throws SQLException, ItemException
  {
    if (hash.containsKey(key(c))) {
      c.delete(this.user);
      localDelete((Cacheable) hash.get(key(c)));
    } else {
      throw new NoSuchItemException("delete: couldn't find item " + key(c));
    }
  }

  /**
    replace an item in the Cache 

    @param c the item to replace (the current item with the same id
      as c will be replaced with c)
    @exception NoSuchItemException if there is no current item with
      the same id as c
    @exception SQLException if there is a problem with the database
    @exception ItemAccessException if the user cannot replace the item
   */
  public void replace(Cacheable c) 
      throws SQLException, NoSuchItemException, ItemAccessException
  {
    if (hash.containsKey(key(c))) {
      c.replace(this.user);
      localDelete((Cacheable) hash.get(key(c)));
      localAdd(c);
    } else {
      throw new NoSuchItemException("replace: couldn't find item " + 
                                    key(c));
    }

    invalidateSort();
  }

  /**
    get an item from the Cache 

    @param id ID of the item
    @return The Cacheable item with the same id
    @exception NoSuchItemException if no item exists with that id
   */
  public Cacheable get(int id) throws NoSuchItemException
  {
    if (hash.containsKey(key(id))) {
      return (Cacheable) hash.get(key(id));
    } else {
      throw new NoSuchItemException("get: couldn't find item " + id);
    }
  }

  /**
    @return If the key refers to an object in the cache
    @param o key to check
   */
  public boolean containsKey(Object o) 
  {
    return hash.containsKey(o);
  }

  /** The elements in the cache, sorted */
  private Vector sorted = null;

  /**
    Invalidate the sorted object
   */
  // synchronized to avoid problems in sortedElements, below
  private synchronized void invalidateSort() 
  {
    sorted = null;
  }

  /**
    Get all the elements in the cache, in sorted order
    @return all the elements in the cache, in sorted order
   */
  // synchronized to avoid race conditions with invalidateSort, above
  public synchronized Iterator sortedElements() 
  {
    if (sorted == null) {
      sorted = new Vector(values());
      Collections.sort(sorted);
    }

    return sorted.iterator();
  }
}
