// java

package edu.washington.cac.myuw.servlet.session;

import java.io.Serializable;

import edu.washington.cac.myuw.servlet.Log;

/** Encapsulates information about a user stored in permanent storage.
  Most data is stored in a cache in the current UWSession.

  @author Greg Barnes
  @version 1.0
 */
public class Profile implements Serializable {
  /** The log associated with this user */
  private Log log = null;

  /** The <code>UWSession</code> we use to store the profile */
  protected UWSession session = null;

  /** The name of the user this profile represent */
  private String userid = null;

    public Profile(UWSession session) {
      this.session = session;

      if (session !=null) {
        userid = session.getUserid();
        session.setProfile(this);
        this.log = session.getLog();
      } else {
        System.err.println("Profile: warning - no session.");
      }
    }

    public void setSession(UWSession session) { 
      this.session = session;

      if (session !=null) {
        this.log = session.getLog();
      } else {
        System.err.println("Profile::setSession warning - no session.");
      }
    }

    public boolean isGuest() {
      return (session != null) ? session.isGuest() : false;
    }

    public String getUserid() {
      return userid;
    }
   
  /** @return the log associated with the Profile */
  public Log getLog() {
    return this.log;
  }

  /* Note: following signature of session.putValue, etc, throw 
     IllegalStateExceptions instead of InvalidSessions if session is null
   */
  /**
   Check that things are okay before doing a put, get or invalidate on
   an object

   @param c object we want to put/get/invalidate 
   @exception IllegalArgumentException if <code>c</code> is bad
   @exception IllegalStateException if the session is bad
   */
  private void checkState(Cacheable c) 
  throws IllegalArgumentException, IllegalStateException
  {
    if (session == null) {
      throw new IllegalStateException("Profile::checkState: session is null!");
    }

    if (c == null) {
      throw new IllegalArgumentException("Profile::checkState: c is null!");
    }

    if (c.getCacheKey() == null) {
      throw new IllegalArgumentException("Profile::checkState: key is null!");
    }
  }

  // implement cachelike functions: put, get, invalidate

  /* standard case where the key is the class of the value we want */
  /**
    Get the only object of type c from the cache
    @param c The type of the object to look for
    @return The object, or null if there is no such object
   */
  public Object get(Cacheable c) 
  {
    checkState(c);
    return get(c.getCacheKey(), c);
  }

      /* Objects are stored in the session variable.  All objects should
         be of type cacheable.  In general, for a particular value v, the
         key for v will be v.getCacheKey().  However, it's possible we might
         want to store two objects of the same class, so we won't insist
         that this is true.

         For some objects, it may be undesirable to save a new value
         (for example, one member of a set of objects is changed, and it
         may be too complex to update the set).  Allow these objects to
         be invalidated, thus forcing a reload the next time we 'get'
         the object */
    public void invalidate(Cacheable c) {
      checkState(c);
      session.removeValue(c.getCacheKey());
    }

      /* standard case where the key of the object we store is the class
          of the object */ 
    protected void put(Cacheable c) 
      throws java.sql.SQLException 
    {
      checkState(c);
      put(c.getCacheKey(), c);
    }

  // Below are private methods:

     /* write new <value>.  This requires saving it in two places:
        permanent storage and the session cache */
    private void put(String key, 
                     Cacheable value) 
      throws java.sql.SQLException 
    {
      value.save();        // save it to permanent storage
      cachePut(key, value);
    }

      /* save the value to the session cache only */
  private void cachePut(String key,
                          Cacheable value) {
    session.putValue(key, value);
  }

      /* Get value from Hashtable referenced by String <key>.  
         Value is of type Cacheable <c>.
         If no such value exists, or if value is expired, 
         load it from permanent storage using
         <c>.  <c> must contain enough information to load the proper
         object given only this Profile object.
        */
    private Object get(String key, 
                       Cacheable c)
    {
        Cacheable co = null;

        co = (Cacheable) session.getValue(key);

        if (co == null) {
          return load(key, c);

        } else if (co.expired()) {

          return load(key, co);

        } else {
          return co;
	}
    }

      /* Like get, except we know the value must be loaded from 
         permanent storage
        */
    private Object load(String key, 
                        Cacheable c) 
    {
        c.setProfile(this);        // give <c> information useful when loading
        Cacheable c2 = c.load();
        cachePut(key, c2); // once we have the object, store it in our cache
        return c2;
    }
}
