package edu.washington.cac.myuw.servlet.session;

import java.util.Comparator;
import java.util.HashMap;
import java.util.Map;

/** Compares two sessions, based on last access time
   @author Greg Barnes
   @version 1.0
 */
/* Note that we use HttpSessionAndId objects instead of HttpSessions, 
   for the reasons noted in SessionManager
 */
public class SessionComparator implements Comparator, java.io.Serializable 
{
  /** Keeps track of last access time for the sessions.

      Each HttpSession keeps track of the time on its own, but we need
      a variable that can be used by a <code>java.util.SortedMap</code>,
      and the access time can change without the SortedMap knowing,
      which will screw up the SortedMap and make Map and SortedMap
      operations work strangely.

      So we will keep another version of last access times available in
      this map, which will not change without our changing it.

      Key = HttpSession's id; Value = last access time for the HttpSession
    */
  private static Map times = new HashMap();

  /**
    @return the last access time of an HttpSession, as a 
      <code>java.lang.Long</code>
    @param session The HttpSession is question
   */
  private static Long lastAccess(HttpSessionAndId session)
  {
    return new Long(session.getSession().getLastAccessedTime());
  }

  /**
    @return the last access time of an HttpSession, as stored in our map.
      Return 0 if the session is not in the map.
    @param id The id of the HttpSession is question
   */
  static Long getLastAccess(String id) 
  {
    if (times.containsKey(id)) {
      return (Long) times.get(id);
    } else {
      return new Long(0);
    }
  }

  /**
    Set the last access time of an HttpSession in our map to the HttpSession's
    current last access time

    @param session The HttpSession is question
   */
  static void setLastAccess(HttpSessionAndId session)
  {
    times.put(session.getId(), lastAccess(session));
  }

  /**
    Remove an HttpSession from our map 
    @param session The HttpSession is question
   */
  static void remove(HttpSessionAndId session)
  {
    times.remove(session.getId());
  }

  /**
    Compare two <code>HttpSessions</code> by comparing their last access times
    in our Map.

    @param o1 The id of one HttpSession to compare
    @param o2 The id of the other HttpSession to compare
    @return An integer less than 0 if o1's access time is before o2's,
      an integer greater than 0 if o1's access time is after o2's, and
      0 only if o1 and o2 are the same session.  

      If the access times of two different sessions are the same 
      (highly unlikely), return an arbitrary (but consistent) non-zero result.  

      If a session is not in the map, the time '0' is used for its last
      access time.
    @exception ClassCastException if either object is not an HttpSession.
   */
  public int compare(Object o1, Object o2) 
  {
    int result = getLastAccess((String) o1).
                 compareTo(getLastAccess(((String) o2)));

    /* by this point, we've thrown a ClassCastException if either object
       is not a HttpSession.  If the result above is somehow zero, just 
       do a String compare on the object's toString values to order non-equal
       sessions with the same access time.  Note this only works if 
       the HttpSessions don't override toString in such a way that
       two different sessions can return the same String.
     */
    return result != 0 ? result : o1.toString().compareTo(o2.toString());
  }
}
