package edu.washington.cac.myuw.util;

import java.net.URLDecoder;

/**
 * This class implements partial URL encoding of strings for insertion into
 * the myuw and uwcal databases, and URL decoding of the strings.
 * <P>
 * By only encoding the characters that give
 * the mango client and its parser problems we simplify the reading of
 * information returned from the database while keeping the increase in 
 * size of the encoded strings to a minimum.
 *
 * TODO:  This class should extend an abstract base class called coder that
 *        specifies the basic operations necessary i.e. encode and decode.
 *        The current Coder class should be renamed to MyUWCoder
 */
public class MyUWURLCoder
{
  public static final String DQ = "%22";
  public static final String SQ = "%27";
  public static final String PCT = "%25";
  public static final String OP = "%28";
  public static final String CP = "%29";
  public static final String BAR = "%7C";
  public static final String TAB = "%09";
  public static final String CR = "%0A";
  public static final String LF = "%0D";
  
  
  public static final char sq = 0x27;
  public static final char dq = 0x22;
  public static final char op = 0x28;
  public static final char cp = 0x29;
  public static final char cr = 0x0A;
  public static final char lf = 0x0D;
  public static final char pct = 0x25;
  public static final char bar = 0x7c;
  public static final char tab = 0x9c; 

  /**
   * Utility method that returns the index of the last character in
   * the input string that will fit in a buffer of the size specified.
   * <P>
   * This method assumes that URL encoding is being used.  As a result
   * of this assumption, every encodable character increases the size
   * of the output string by 2 bytes, hence the appearance of the 
   * magic number 2 in the body of the method
   *
   *@param s the input string
   *@param bufferSize the buffer size
   *@return the index of the leftmost character in
   *  <code>s</code> that will fit in a buffer 
   *  of size <code>bufferSize</code> after it is encoded
   */
  public static int fitStringToBuffer(String s, int bufferSize)
    throws IllegalArgumentException 
  {   
    int i, outSize = 0;

    if( s == null ) {
      throw new IllegalArgumentException(
        "MyUWURLCoder: fitStringToBuffer, null argument");
    }
      
    for(i=0; i<s.length(); i++) {
      if( isEncodeable(s.charAt(i) )) {
        outSize += 3;
      } else {
        outSize++;
      }

      if (outSize > bufferSize) {
        return i-1;     
      }
    }

      // last char in s will fit; its index is s.length() - 1
    return s.length() - 1;   
  }

  /** Return true if the specified character is one that needs to be encoded */
  public static boolean isEncodeable( char ch )
  {
    boolean result;

    switch(ch)
    {
      case sq:
      case dq:
      case op:
      case cp:
      case cr:
      case lf:
      case pct:
      case tab:
      case bar:
        result = true;
        break;
      
      default:
        result = false;
        break;
    }

    return result;
  }
 

  public static String encode( String in )
  {
    char ch;
    StringBuffer out = new StringBuffer();
    
    for(int i=0;i<in.length();i++)
      {
	ch=in.charAt(i);
	
	switch(ch)
	  {
	  case sq:
	    out.append(SQ);
	    break;
	  case dq:
	    out.append(DQ);
	    break;
	  case op:
	    out.append(OP);
	    break;
	  case cp:
	    out.append(CP);
	    break;
	  case cr:
	    out.append(CR);
	    break;
	  case lf:
	    out.append(LF);
	    break;
	  case pct:
	    out.append(PCT);
	    break;
	  case tab:
	    out.append(TAB);
	    break;
	  case bar:
	    out.append(BAR);
	    break;
	  default:
            out.append(ch);
            break;
	  }
      }

    return out.toString();
  }
  
  /** since we are using a subset of URLEncoding, the Standard URLDecoder
    * functions quite well so we wrap it in our decode method
    */
  public static String decode( String str )
  {
    return URLDecoder.decode(str); 
  }

  public static void main( String args[] )
  {
    String s1="This one i";
    String s2="This one doesn't fit";
    String s3="(encod%d)";
    String s4="this%";
    String s5="(asdf)";
    
    int idx = fitStringToBuffer(s1,10);
    System.out.println("Idx is: " + idx + " String is: " + s1 +
		       " Last char to fit is: " + s1.charAt(idx));
    idx = fitStringToBuffer(s2,10);
    System.out.println("Idx is: " + idx + " String is: " + s2+
		       " Last char to fit is: " + s2.charAt(idx));
    idx = fitStringToBuffer(s3,10);
    System.out.println("Idx is: " + idx + " String is: " + s3+
		       " Last char to fit is: " + s3.charAt(idx)); 
    idx = fitStringToBuffer(s4,10);
    System.out.println("Idx is: " + idx + " String is: " + s4+ 
		       " Last char to fit is: " + s4.charAt(idx));
    idx = fitStringToBuffer(s5,10);
    System.out.println("Idx is: " + idx + " String is: " + s5+
		       " Last char to fit is: " + s5.charAt(idx));
    
    System.out.println("");
    
    System.out.println("Original String: " + s1); 
    System.out.println("Encoded String: " + encode(s1));
    System.out.println("Decoded StringL " + decode(s1));
    System.out.println("Original String: " + s2); 
    System.out.println("Encoded String: " + encode(s2));
    System.out.println("Decoded StringL " + decode(s2));
    System.out.println("Original String: " + s3); 
    System.out.println("Encoded String: " + encode(s3));
    System.out.println("Decoded StringL " + decode(encode(s3)));
    System.out.println("Original String: " + s4); 
    System.out.println("Encoded String: " + encode(s4));
    System.out.println("Decoded StringL " + decode(encode(s4)));
    System.out.println("Original String: " + s5); 
    System.out.println("Encoded String: " + encode(s5));
    System.out.println("Decoded StringL " + decode(encode(s5)));  
  }
}
