package pubevents.extractdata;

import java.io.PrintWriter;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.text.SimpleDateFormat;
import java.util.Calendar;

import org.apache.ecs.StringElement;
import org.apache.ecs.html.B;
import org.apache.ecs.html.BR;

import edu.washington.cac.calendar.MyCalendar;
import edu.washington.cac.calendar.db.Fields;
import uwcal.out.DisplayText;
import uwcal.out.forms.FormsDisplayText;

/** Prints an event
  
  @author Greg Barnes
  @version 1.0
 */
public class EventPrinter
{
  /** Writer to print to */
  public PrintWriter out;

  /** Result set, currently pointing to the event to print */
  public ResultSet rs;

  /** Create a new EventPrinter
    @param out Writer to print to 
    @param rs Result set, currently pointing to the event to print 
   */
  EventPrinter(PrintWriter out, ResultSet rs)
  {
    this.out = out;
    this.rs = rs;
  }

  /** An HTML BR element, as a string */
  public static final String BR = new BR().toString();

  /** @return a fieldName whose text is s */
  public static String fieldName(String s)
  {
    return new B(s + ": ").toString();
  }

  /** The months, as dictated by the AP style manual */
  public static final String[] AP_MONTH = 
  {
    "Jan.", "Feb.", "March", "Apr.", "May", "June", "July", "Aug.",
    "Sept.", "Oct.", "Nov.", "Dec."
  };

  /** Print the starting date of the event
    @exception SQLException if there's trouble getting the data from the 
      result set
   */
  public void printStartdate() throws SQLException
  {
    MyCalendar cal = 
        new MyCalendar(this.rs.getDate(Fields.ESTARTDATE.getName()));
    this.out.print(fieldName(FormsDisplayText.START_DATE));
    this.out.println(cal.timeString(new SimpleDateFormat(
        "EEEE, '" + AP_MONTH[cal.get(Calendar.MONTH)] + "' d")));
    this.out.println(BR);
  }

  /** Print the dates of the event
    @exception SQLException if there's trouble getting the data from the 
      result set
   */
  public void printDates() throws SQLException
  {
    this.out.print(fieldName(FormsDisplayText.START_DATE + " - " 
                             + FormsDisplayText.END_DATE));
    this.out.println(rs.getDate(Fields.ESTARTDATE.getName()) + " - " + 
                     rs.getDate(Fields.EENDDATE.getName()));
    this.out.println(BR);
  }

  /** Print the (1st) category of the event
    @exception SQLException if there's trouble getting the data from the 
      result set
   */
  public void printCategory() throws SQLException
  {
    this.out.print(fieldName("Category"));
    this.out.println(this.rs.getString(Fields.KWORD.getName()).
                         replace('_', ' '));
    this.out.println(BR);
  }

  /** Print the short description of the event
    @exception SQLException if there's trouble getting the data from the 
      result set
   */
  public void printShortdesc() throws SQLException
  {
    this.out.print(fieldName(FormsDisplayText.NAME));
    this.out.println(this.rs.getString(Fields.ESHORTDESC.getName()));
    this.out.println(BR);
  }

  /** Print the long description of the event
    @exception SQLException if there's trouble getting the data from the 
      result set
   */
  public void printLongdesc() throws SQLException
  {
    this.out.print(fieldName(FormsDisplayText.LONG_DESCRIPTION));
    this.out.println(Fields.getEventLongdesc(this.rs));
    this.out.println(BR);
  }

  /** A time format that includes minutes */
  public static final SimpleDateFormat TIME_MIN = 
      new SimpleDateFormat("h:mm");

  /** A time format that includes the hour only */
  public static final SimpleDateFormat TIME = 
      new SimpleDateFormat("h");

  /**
    @param cal A calendar possibly containing a time
    @return A string representation of the calendar's am/pm field
   */
  public String ampm(MyCalendar cal)
  {
    return (cal.get(Calendar.AM_PM) == Calendar.AM) ? "a.m." : "p.m.";
  }

  /** @return Does the calendar have a minute value other than 0? */
  public boolean hasMinutes(MyCalendar cal) 
  {
    return cal.get(Calendar.MINUTE) != 0;
  }

  /**
    @param cal A calendar possibly containing a time
    @return A string represenation of the calendar's time
    @param showAmpm whether to display am/pm
   */
  public String time(MyCalendar cal, boolean showAmpm) 
  {
    if (cal.get(Calendar.HOUR) % 12 == 0 && !hasMinutes(cal)) {
      return (cal.get(Calendar.AM_PM) == Calendar.AM) ? "midnight" : "noon";
    } else {
      return cal.timeString(hasMinutes(cal) ? TIME_MIN : TIME) + 
             (showAmpm ? " " + ampm(cal) : "");
    }
  }

  /** Print the time of the event
    @exception SQLException if there's trouble getting the data from the 
      result set
   */
  public void printTime() throws SQLException
  {
    if (this.rs.getTime(Fields.ESTARTTIME.getName()) == null
        && this.rs.getTime(Fields.EENDTIME.getName()) == null) 
    {
      return;
    }

    this.out.print(fieldName(FormsDisplayText.TIME));

    MyCalendar startCal =
        new MyCalendar(this.rs.getDate(Fields.ESTARTDATE.getName()),
                       this.rs.getTime(Fields.ESTARTTIME.getName()));

    if (this.rs.getTime(Fields.EENDTIME.getName()) != null) 
    {
      MyCalendar endCal =
          new MyCalendar(this.rs.getDate(Fields.EENDDATE.getName()),
                         this.rs.getTime(Fields.EENDTIME.getName()));
      this.out.print(time(startCal, 
          startCal.get(Calendar.AM_PM) != endCal.get(Calendar.AM_PM)
          || endCal.get(Calendar.HOUR) % 12 ==  0));
      this.out.print("-");
      this.out.print(time(endCal, true));
    } else {
      this.out.print(time(startCal, true));
    }

    this.out.println(BR);
  }

  /** Print the location of the event
    @exception SQLException if there's trouble getting the data from the 
      result set
   */
  public void printLocation() throws SQLException
  {
    if (this.rs.getString(Fields.LADDRESS.getName()) != null &&
        !this.rs.getString(Fields.LADDRESS.getName()).
            equalsIgnoreCase(DisplayText.NO_VALUE))
    {
      this.out.print(fieldName(FormsDisplayText.LOCATION));
      this.out.print(this.rs.getString(Fields.LADDRESS.getName()));
      this.out.println(BR);
    }
  }


  /** Print the cost of the event
    @exception SQLException if there's trouble getting the data from the 
      result set
   */
  public void printCost() throws SQLException
  {
    if (this.rs.getString(Fields.ECOST.getName()) != null &&
        !this.rs.getString(Fields.ECOST.getName()).equals(""))
    {
      this.out.print(fieldName("Cost"));
      this.out.print(this.rs.getString(Fields.ECOST.getName()));
      this.out.println(BR);
    }
  }

  /** Print the link for the event
    @exception SQLException if there's trouble getting the data from the 
      result set
   */
  public void printLink() throws SQLException
  {
    if (this.rs.getString(Fields.ELINK.getName()) != null &&
        !this.rs.getString(Fields.ELINK.getName()).equals(""))
    {
      this.out.print(fieldName(FormsDisplayText.EVENT_LINK));
      this.out.print(this.rs.getString(Fields.ELINK.getName()));
      this.out.println(BR);
    }
  }

  /** Print the link for the event
    @exception SQLException if there's trouble getting the data from the 
      result set
   */
  public void printLocationLink() throws SQLException
  {
    if (this.rs.getString(Fields.LLINK.getName()) != null &&
        !this.rs.getString(Fields.LLINK.getName()).equals(""))
    {
      this.out.print(fieldName(FormsDisplayText.LOCATION_LINK));
      this.out.print(this.rs.getString(Fields.LLINK.getName()));
      this.out.println(BR);
    }
  }

  /** Print the sponsor of the event
    @exception SQLException if there's trouble getting the data from the 
      result set
   */
  public void printSponsor() throws SQLException
  {
    if (rs.getString(Fields.SNAME.getName()) != null &&
        !rs.getString(Fields.SNAME.getName()).
            equalsIgnoreCase(DisplayText.NO_VALUE))
    {
      this.out.print(fieldName(DisplayText.SPONSOR));
      this.out.print(this.rs.getString(Fields.SNAME.getName()));
      this.out.println(BR);
    }
  }

  /** Print the phone number of the sponsor of the event
    @exception SQLException if there's trouble getting the data from the 
      result set
   */
  public void printSponsorPhone() throws SQLException
  {
    if (rs.getString(Fields.SPHONE.getName()) != null &&
        !rs.getString(Fields.SPHONE.getName()).equals(""))
    {
      this.out.print(fieldName(DisplayText.SPONSOR + " " + 
                               FormsDisplayText.PHONE));
      this.out.print(rs.getString(Fields.SPHONE.getName()));
      this.out.println(BR);
    }
  }

  /** Print the link for the event
    @exception SQLException if there's trouble getting the data from the 
      result set
   */
  public void printSponsorLink() throws SQLException
  {
    if (this.rs.getString(Fields.SLINK.getName()) != null &&
        !this.rs.getString(Fields.SLINK.getName()).equals(""))
    {
      this.out.print(fieldName(FormsDisplayText.SPONSOR_LINK));
      this.out.print(this.rs.getString(Fields.SLINK.getName()));
      this.out.println(BR);
    }
  }

  /** Print the link for the event
    @exception SQLException if there's trouble getting the data from the 
      result set
   */
  public void printSponsorEmail() throws SQLException
  {
    if (this.rs.getString(Fields.SEMAIL.getName()) != null &&
        !this.rs.getString(Fields.SEMAIL.getName()).equals(""))
    {
      this.out.print(fieldName(DisplayText.SPONSOR + " " +
                               FormsDisplayText.EMAIL));
      this.out.print(this.rs.getString(Fields.SEMAIL.getName()));
      this.out.println(BR);
    }
  }
}
