// java

package uwcal.out;

import java.util.Iterator;

import org.apache.ecs.AlignType;
import org.apache.ecs.Element;
import org.apache.ecs.ElementContainer;
import org.apache.ecs.Entities;
import org.apache.ecs.html.BR;
import org.apache.ecs.html.HR;
import org.apache.ecs.html.LI;
import org.apache.ecs.html.Span;
import org.apache.ecs.html.Table;
import org.apache.ecs.html.TD;
import org.apache.ecs.html.TR;
import org.apache.ecs.html.UL;

import edu.washington.cac.calendar.CalParameter;
import edu.washington.cac.calendar.CalProperties;
import edu.washington.cac.calendar.MyCalendar;
import edu.washington.cac.calendar.data.EventsI;
import edu.washington.cac.calendar.data.PersonalEvents;
import edu.washington.cac.calendar.data.User;
import edu.washington.cac.calendar.filter.Calendar;
import edu.washington.cac.calendar.filter.Filter;
import edu.washington.cac.calendar.filter.PublicCalendars;

import uwcal.out.forms.CommonFormElements;
import uwcal.out.forms.FormsDisplayText;

/**
  A page with all the named calendars
  @author Greg Barnes
  @version 2.0
 */
/* To make for a more generic page, instead of having only 2 classes of
   calendars (main, default), we should allow for any number of
   'top-level' set.  See PublicCalendars for more discussion.
 */
public class AllCalendars extends IntermediatePage
{
  /** An object that holds the main list of calendars */
  protected CalendarSet mcl;

  /** A helper object to print rows, headers, etc. */
  protected CommonFormElements cfe;

  /**
    Create a new instance
    @param es User's events cache
    @param message Message to display at the top of the page
    @param view View we came from
   */
  protected AllCalendars(EventsI es, 
                         String message,
                         Timeview view) 
  {
    super(es, message, view);
    this.mcl = new CalendarSet(mc, view, PublicCalendars.mainCalendars());
    this.cfe = new CommonFormElements();
  }

  /**
    Get the number of filters represented by this filter 
    (1 + number of subfilters).
    @param filter the filter
    @return the number of filters represented by this filter
   */
  private int numberOfFilters(Filter filter)
  {
    if (filter.showChildren()) {
      Iterator i = filter.getChildren().iterator();
      int subfilters = 0;

      while (i.hasNext()) {
        subfilters += numberOfFilters(((Calendar) i.next()).getFilter());
      }

      return subfilters + 1;
    } else {
      return 1;
    }
  }

  /**
    Get the number of total calendars represented by the list of main calendars
    @param calendars the list of main calendars
    @return the number of total calendars
   */
  private int totalCalendars(Calendar[] calendars)
  {
    int totalCalendars = 0;

    for (int i = 0; i < calendars.length; i++) {
      totalCalendars += numberOfFilters(calendars[i].getFilter());
    }

    return totalCalendars;
  }

  /** Get a calendar's subcalendars, as HTML
    @param calendar The calendar
    @param indentation Base indentation to appear before subcalendars' names
    @return the subcalendars, as HTML
   */
  protected Element subCalendars(Calendar calendar, Element indentation)
  {
    Filter filter = calendar.getFilter();

    if (filter.showChildren()) {
      Iterator i = filter.getChildren().iterator();
      ElementContainer ec = new ElementContainer();

      while (i.hasNext()) {
        ec.addElement(format((Calendar) i.next(), indentation, false));
      }

      return ec;
    } else {
      return new ElementContainer();
    }
  }

  /** 
    Get a blank row 
    @return a blank row 
   */
  private TR blankRow()
  {
    return new TR().addElement(new TD().addElement(Entities.NBSP));
  }

  /**
    Get the contents of a single calendar row as HTML
    @param cal Calendar
    @return the calendar, as HTML
   */
  protected Element oneCalendar(Calendar cal)
  {
    return new ElementContainer().addElement(this.mcl.name(cal)).
                                  addElement("\n");
  }

  /**
    Get a calendar and all its subcalendars, as HTML
    @param cal Calendar
    @param indentation Indentation to appear before the calendar name 
    @param isTopLevel Is this a top level calendar?
    @return the calendar and all its subcalendars, as HTML
   */
  private Element format(Calendar cal, 
                         Element indentation, 
                         boolean isTopLevel)
  {
    return // top level calendar
        new ElementContainer().
            addElement(new TR().
                addElement(new TD().addElement(indentation).
                                    addElement(oneCalendar(cal))).
                setClass(isTopLevel ? Style.TOPCALENDAR_ROW : 
                                      Style.SUBCALENDAR_ROW)).
            addElement(subCalendars(cal, 
                isTopLevel ? indentation : 
                             new ElementContainer(indentation).
                                 addElement(DisplayText.MANY_NBSP))).
            addElement(isTopLevel ? (Element) blankRow() : 
                                    new ElementContainer());
  }

  /**
    Turn a column table into a column cell
    @param table Column table
    @return The table, inserted into a column cell
   */
  private Element columnCell(Table table)
  {
    return new TD().addElement(table).setClass(Style.CALENDAR_LIST_COLUMN);
  }

  /**
    Get a list of calendars, and their subcalendars, as two columns
    in a table
    @return the list of calendars and their subcalendars
   */
  protected Element twoColumnCalendarList(CalendarSet cs)
  {
    Table leftColumn = null;      // the left column
    Table currentColumn = new Table();
    currentColumn.setClass(Style.CALENDAR_LIST_TABLE);

    Calendar[] calendars = cs.getCalendarArray();
    int columnCount = 0;        // number of calendars in the current column

    for (int i = 0; i < calendars.length; i++) {
      if (leftColumn == null && 
          columnCount >= (totalCalendars(calendars) + 1)/2) 
      { // done with the left column
        leftColumn = currentColumn;
        currentColumn = new Table();
        currentColumn.setClass(Style.CALENDAR_LIST_TABLE);
      }

      currentColumn.addElement(format(calendars[i], 
                                      new ElementContainer(), true));
      columnCount += numberOfFilters(calendars[i].getFilter());
    }

    currentColumn.addElement(format(PublicCalendars.defaultCalendar(),
                                    new ElementContainer(), true));

    if (leftColumn == null) {   
      // could happen if last calendar has more than half the calendars
      return cfe.tableRowTwoCells(columnCell(currentColumn), 
                                  new ElementContainer());
    } else {
      return cfe.tableRowTwoCells(columnCell(leftColumn), 
                                  columnCell(currentColumn));
    }
  }

  /**
    Get the list of main categories as HTML
    @return the list of main categories as HTML
   */
  protected Element allCalendars()
  {
    return new ElementContainer().
           addElement(cfe.headerRow(DisplayText.ALLCALENDARS_HEADER,
                                    Style.FORM_TITLE)).
           addElement("\n").
           addElement(twoColumnCalendarList(mcl)).
           addElement("\n");
  }

  /**
    @return A url that will yield the allcalendars page 
    @param mc Current date for the page
    @param page Page this URL is enclosed in
   */
  public static String url(MyCalendar mc, Page page) 
  {
    return baseUrl(CalParameter.ALLCALENDARS_FUNC, null) + 
        mc.getDateDigits() + Util.FUNCTION2_PARAM + page.nextFunction();
  }

  protected Element contents() 
  {
    return new Table().setCellSpacing("0").setCellPadding("0").
               addElement("\n").
               addElement(allCalendars()).
               setAlign(AlignType.CENTER).
               setID(Style.SUBSCRIPTIONS_TABLE);
  }

  protected String title() 
  {
    return DisplayText.ALLCALENDARS_TITLE;
  }

  /**
    Should this page show the icon legend for guests?
    @return Should this page show the icon legend for guests?
   */
  protected boolean shouldShowIconLegend()
  {
    return false;
  }

  /**
    Should this page display the filter message?
    @return Should this view display the filter message?
   */
  protected boolean shouldShowFilterMessage()
  {
    return false;
  }
}
