// java

package uwcal.out;

import java.io.IOException;
import java.sql.SQLException;
import java.util.Date;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletResponse;

import org.apache.ecs.Document;
import org.apache.ecs.Element;
import org.apache.ecs.ElementContainer;
import org.apache.ecs.html.Body;
import org.apache.ecs.html.BR;
import org.apache.ecs.html.Center;
import org.apache.ecs.html.I;
import org.apache.ecs.html.IMG;
import org.apache.ecs.html.Span;
import org.apache.ecs.html.Table;
import org.apache.ecs.html.TD;
import org.apache.ecs.html.TR;

import edu.washington.cac.calendar.CalParameter;
import edu.washington.cac.calendar.CalProperties;
import edu.washington.cac.calendar.MyCalendar;
import edu.washington.cac.calendar.data.CaldataException;
import edu.washington.cac.calendar.data.EventsI;
import edu.washington.cac.calendar.data.ItemException;
import edu.washington.cac.calendar.data.User;
import edu.washington.cac.calendar.filter.FilteredEvents;

import uwcal.out.forms.GoForm;

/**
  An output web page in the calendar application
  @author Greg Barnes
  @version 2.2
 */
public abstract class Page 
{
  /** The current date for the page */
  protected MyCalendar mc = null;

  /** The user of the application */
  protected User user;

  /** The user's events */
  protected EventsI es;

  /** A message for the user to see */
  protected String message = null;

  protected Page(MyCalendar mc, User user, EventsI es, String message) 
  {
    this(mc, user, es);
    this.message = message;
  }

  protected Page(MyCalendar mc, User user, EventsI es) 
  {
    this.mc = mc;
    this.user = user;
    this.es = es;
  }
  
  /**
    Get a calendar corresponding to the start date in the HTTP parameters
    @param cp Contains the HTTP parameters
    @param dfault A default calendar to use if we can't find a startdate
      in <code>cp</code>
    @returns The calendar corresponding to the start date
   */
  // Note: can't call 'dfault' 'default', as 'default' is a reserved word
  private static MyCalendar getStartCalendar(CalParameter cp, 
                                             MyCalendar dfault) 
  {
    try {
      return cp.getStartdate() == null ? dfault : 
        new MyCalendar(cp.getStartdate());
    } catch (CaldataException e) {
      return dfault;
    }
  }

  /** factory methods to create a Page corresponding to a particular 
      function

    @param function The function to perform
    @param mc The current date
    @param user The user
    @param es The list of events
    @param cp Any form parameters
    @return a page for the given function

    @exception ItemException if the page is supposed to perform
      an operation on an item, but it cannot
   */
  public static Page create(String function, 
                            MyCalendar mc, 
                            User user, 
                            EventsI es, 
                            String message,
                            CalParameter cp) 
      throws ItemException
  {
    if (function.equals(CalParameter.DAYVIEW_FUNC)) {
      return new Dayview(mc, user, es, message);
    } else if (function.equals(CalParameter.WEEKVIEW_FUNC)) {
      return new Weekview(mc, user, es, message, 
                          !CalProperties.defaultIsGuest());
    } else if (function.equals(CalParameter.MONTHVIEW_FUNC)) {
      return new Monthview(mc, user, es, message,
                           !CalProperties.defaultIsGuest());
    } else if (function.equals(CalParameter.YEARVIEW_FUNC)) {
      return new Yearview(mc, user, es, message);
    } else if (function.equals(CalParameter.EVENTVIEW_FUNC)) {
      return new Eventview(es, message, 
          Timeview.createView(cp.getFunction2(), mc, user, es, message, cp),
          es.getEvent(cp.getEventID()));
    } else if (function.equals(CalParameter.GO_FUNC)) {
      return new Dayview(getStartCalendar(cp, mc), user, es, message);
    } else if (function.equals(CalParameter.ALLCALENDARS_FUNC)) {
      return CalProperties.defaultIsGuest() ?
             new AllCalendars(es, message,
                 Timeview.createView(cp.getFunction2(), mc, user, es, 
                                     message, cp)) :
             new Subscriptions(es, message,
                 Timeview.createView(cp.getFunction2(), mc, user, es, 
                                     message, cp));
    } else if (function.equals(CalParameter.EDITEVENT_FUNC)) {
      return new EditEvent(es, message, 
          Timeview.createView(cp.getFunction2(), mc, user, es, message, cp),
          es.getEvent(cp.getEventID()));
    } else if (function.equals(CalParameter.ENTERKEYWORD_FUNC)) {
      return new AddKeyword(es, message, 
          Timeview.createView(cp.getFunction2(), mc, user, es, message, cp));
    } else if (function.equals(CalParameter.EDITKEYWORD_FUNC)) {
      return new EditKeyword(es, message, 
          Timeview.createView(cp.getFunction2(), mc, user, es, message, cp),
          es.getKeywords().getKeyword(cp.getKeywordID()));
    } else if (function.equals(CalParameter.ENTERSPONSOR_FUNC)) {
      return new AddSponsor(es, message, 
          Timeview.createView(cp.getFunction2(), mc, user, es, message, cp));
    } else if (function.equals(CalParameter.EDITSPONSOR_FUNC)) {
      return new EditSponsor(es, message, 
          Timeview.createView(cp.getFunction2(), mc, user, es, message, cp),
          es.getSponsors().getSponsor(cp.getKeywordID()));
    } else if (function.equals(CalParameter.ENTERLOCATION_FUNC)) {
      return new AddLocation(es, message, 
          Timeview.createView(cp.getFunction2(), mc, user, es, message, cp));
    } else if (function.equals(CalParameter.EDITLOCATION_FUNC)) {
      return new EditLocation(es, message, 
          Timeview.createView(cp.getFunction2(), mc, user, es, message, cp),
          es.getLocations().getLocation(cp.getLocationID()));
    } else {
      throw new IllegalArgumentException("Page::create: illegal function");
    }
  }

  /** 
    @return the main part of the page as HTML
    @exception SQLException if we have to access the database and
      have a problem
    @exception CaldataException if we have to access the database,
      and it contains bad data
   */
  protected abstract Element contents() throws SQLException, CaldataException;

  protected abstract String title();

  /** The prefix for all URLs generated by this class and its subclasses.
      Should be a relative URL
   */
  private static final String PREFIX = WebResources.BASE_GET_URL;

  /** 
    @return a base URL, up to the date 
    @param func function for the URL
    @param prefix prefix to use instead of the default
      null to use the default
   */
  static String baseUrl(String func, String prefix) 
  {
    return (prefix == null ? PREFIX : prefix) + func + Util.DATE_PARAM;
  }

  /**
    Get the object that can produce the prev/this/next links
    @return the object that can produce the prev/this/next links
   */
  protected abstract DateHeader getDateHeader();

  /**
    Get the function name for the page you see most often when you follow
    a link from this page
    @return the function name for the page you see most often when you follow
    a link from this page
   */
  public abstract String nextFunction();

  /**
    Get the prev/this/next links
    @return the prev/this/next links
   */
  private Element prevThisNextLinks() 
  {
    return getDateHeader().getHeader();
  }

  private static Element infoLinks() {
      return new ElementContainer().
        addElement(Util.linkItem(WebResources.ABOUT_URL,
          DisplayText.ABOUT_TEXT)).
        addElement(" | ").
        addElement(Util.linkItem(WebResources.HELP_URL,
          DisplayText.HELP_TEXT));
  }

  private Element goForm() {
      return new GoForm(this).getForm();
  }

    // by default, these are all false.  Override them to make them true
  protected boolean isYearview() {
      return false;
  }

  protected boolean isMonthview() {
      return false;
  }

  protected boolean isWeekview() {
      return false;
  }

  protected boolean isDayview() {
      return false;
  }

  protected boolean isToday() {
      return false;
  }

  /**
    Get the HTML in a navigation button
    @param link URL for the button
    @param text Text of the button
    @param isOn Is the button 'on'?
   */
  private static Element navButtonText(String link, String text, boolean isOn) 
  {
    return Util.linkItem(link, text, 
        isOn ? Style.BUTTON_ON : Style.BUTTON_OFF, null);
  }

  /**
    Get a navigation button
    @param link URL for the button
    @param text in the button
    @param isOn Is the button 'on'?
    @return A navigation button
   */
  private static Element navButton(String link, String text, boolean isOn) 
  {
    return new TD(navButtonText(link, text, isOn)).
        setClass(isOn ? Style.NAVBUTTON_ON : Style.NAVBUTTON_OFF);
  }

  private Element yearButton() {
    return navButton(Yearview.url(mc), DisplayText.YEARVIEW, isYearview());
  }

  private Element monthButton() {
    return navButton(Monthview.url(mc), DisplayText.MONTHVIEW, 
        isMonthview());
  }

  private Element weekButton() {
    return navButton(Weekview.url(mc), DisplayText.WEEKVIEW, isWeekview());
  }

  private Element dayButton() {
    return navButton(Dayview.url(mc), DisplayText.DAYVIEW, isDayview());
  }

  private Element todayButton() {
    return navButton(Dayview.url(new MyCalendar()), DisplayText.TODAY,
        isToday());
  }

  private Element navButtonTable() {
      return new Table().
        addElement(new TR().
          addElement(todayButton()).addElement("\n").
          addElement(dayButton()).addElement("\n").
          addElement(weekButton()).addElement("\n").
          addElement(monthButton()).addElement("\n").
          addElement(yearButton()).addElement("\n").
          setID(Style.NAVBUTTON_ROW)).
        setCellSpacing(0).setCellPadding(1).
setWidth("100%").setBorder(1).                //%%% for non-compliant browsers
        setID(Style.NAVBUTTON_TABLE);
  }

    // number of cells in the button row
  protected static final int BUTTONROW_CELLS = 3;
 
  /**
    Should we see the middle row of links in the header?
    @return Should we see the middle row of links in the header?
       Default is true.
   */
  protected boolean shouldShowButtonRow()
  {
    return true;
  }

  /**
    Get all the links in the middle row of the header
    @return all the links in the middle row of the header
   */
  private Element buttonRow() 
  {
    if (!shouldShowButtonRow()) {
      return new ElementContainer();
    } else {
      return new TR().
             addElement("\n").
             addElement(prevThisNextLinks()).addElement("\n").   
             addElement(new TD(goForm()).setNoWrap(true).
                        setID(Style.GOTO_CELL)).
             addElement("\n").
             addElement(new TD(navButtonTable()).setNoWrap(true).
setWidth("10%").                //%% for non-compliant browsers
                        setID(Style.NAVBUTTON_CELL));
    }
  }

  private static final Element LOGO = new IMG(WebResources.LOGO_URL).
//%%% for non-compliant browsers
setWidth(Style.LOGO_WIDTH).setHeight(Style.LOGO_HEIGHT).  
    setAlt(DisplayText.LOGO_ALT).setID(Style.LOGO_IMAGE);

  private static Element infoCell() 
  {
    return new TD().setColSpan(BUTTONROW_CELLS).
                    addElement("\n").addElement(new Span().
                    addElement(infoLinks()).setClass(Style.AUX)).
                    setClass(Style.HEADER_LINKS);
  }

  private Element INFO_ROW = 
      new TR().addElement("\n").addElement(infoCell());

  /**
    the 'tool' links
    @return the 'tool' links
   */
  private Element toolLinks()
  {
    return this.user.isGuest() ?
       Util.linkItem(WebResources.pubeventsEntryUrl(),
                     DisplayText.SUBMIT_PUBLIC_EVENT_TEXT) :
// currently, tool link only shows up on time views in personal calendar
       new ElementContainer().
           addElement(Util.linkItem(AddLocation.url(mc, this), 
                                    DisplayText.ADD_LOCATION_TEXT)).
           addElement(" | ").
           addElement(Util.linkItem(AllCalendars.url(mc, this), 
                                    DisplayText.SUBSCRIPTIONS_TEXT));
  }

  /**
    Should this page include the row of 'tool' links?
    @return Should this page include the row of 'tool' links?
   */
  protected boolean shouldShowToolRow()
  {
    return this.user.isGuest();
  }

  /**
    Get the row of 'tool' links
    @return The row of 'tool' links
   */
  private Element toolRow() 
  {
    if (shouldShowToolRow()) {
      return new TR().
          addElement(new TD().setColSpan(BUTTONROW_CELLS).
                     addElement("\n").
                     addElement(new Span().
                                addElement(toolLinks()).
                                setClass(Style.AUX)).
                     setClass(Style.HEADER_LINKS));
    } else {
      return new ElementContainer();
    }
  }

  /**
    Should this page display the filter message?
    @return Should this view display the filter message?
   */
  protected boolean shouldShowFilterMessage()
  {
    return false;
  }

  /**
    Get the messages to display at the top of the page
    @return The messages, as HTML
   */
  protected Element messages() 
  {
    // Add information about current filters for guest users
    if (this.user.isGuest() && shouldShowFilterMessage()) {
      this.message = DisplayText.addMessage(
          this.message,
          new FilterMessage(((FilteredEvents) this.es).getFilter()).
              getMessage());
    }

    if (this.message == null || this.message.trim().length() == 0) { 
      return new ElementContainer();
    } else { 
      return new TR().addElement(new Message(this.message).asTableCell());
    }
  }

  /**
    Get all the links in the header, as a table
    @return all the links in the header
   */
  private Element links() 
  {
    return new Table().setCellSpacing(0).setCellPadding(0).
setWidth("100%").                //%%% for non-compliant browsers
           addElement("\n").
           addElement(INFO_ROW).addElement("\n").
           addElement(buttonRow()).addElement("\n").
           addElement(toolRow()).addElement("\n").
           addElement(messages()).addElement("\n").
           setID(Style.RIGHTHEADER_TABLE);
  }

  /**
    Get the header for the page
    @return the header for the page
   */
  private Element header() 
  {
    return new Table().setCellSpacing(0).setCellPadding(0).
setWidth("100%").        //%%% for non-compliant browsers
               addElement("\n").
               addElement(new TR().
                          addElement(new TD(LOGO).
setWidth("10%").        //%%% for non-compliant browsers
                                     setID(Style.LOGO_CELL)).
                          addElement("\n").
                          addElement(new TD(links()).
setWidth("90%").        //%%% for non-compliant browsers
                                     setID(Style.RIGHTHEADER_CELL)).
                          addElement("\n")).
               setID(Style.HEADER_TABLE);
  }

  /**
    Should this page show the icon legend for guests?
    @return Should this page show the icon legend for guests?
   */
  protected boolean shouldShowIconLegend()
  {
    return true;
  }

  /** 
    Get the legend at the bottom.
        For users, says who is logged in 
        For guests, explains the add icon, if necessary
    @return The legend
    */
  protected Element legend() 
  {
    if (this.user.isGuest()) {
      if (shouldShowIconLegend()) {
        return new ElementContainer().
               addElement(AddEventRef.iconLegend());
      } else {
        return new ElementContainer();
      }
    } else {
      return new ElementContainer().
             addElement(new I(DisplayText.LOGGED_IN_TEXT)).
             addElement(this.user.getNameDB());
    }
  }

  protected Element footer() {
      return new ElementContainer().addElement(new BR()).
        addElement(new Center().addElement(legend())).addElement("\n").
        addElement(DisplayText.footer);
  }

  /**
    Get the body of the page
    @return the body
    @exception SQLException if have to access the database, and
      there is a problem
    @exception CaldataException if we have to access the database,
      and it contains bad data
   */
  private Element body() throws SQLException, CaldataException
  {
    return new ElementContainer().addElement("\n").
               addElement(header()).addElement("\n").
               addElement(contents()).addElement("\n").
               addElement(footer()).addElement("\n");
  }

  /**
    Get the page
    @return the page, as a Document
    @exception SQLException if have to access the database, and
      there is a problem
    @exception CaldataException if we have to access the database,
      and it contains bad data
   */
  private Document getDocument() throws SQLException, CaldataException
  {
    return new Document().appendTitle(title()).appendHead("\n").
               appendHead(DisplayText.COPYRIGHT).appendHead("\n").
               appendHead(Style.STYLE_SHEET).
               setBody(new Body().
//setBgColor(Style.DARKER_GRAY).  //%%%% for non-compliant browsers
                       addElement(body()));
  }

  /**
    Print the page to the servlet's output stream
    @param resp The servlet's Response object
    @exception IOException if there's a problem printing
    @exception SQLException if have to access the database, and
      there is a problem
    @exception CaldataException if we have to access the database,
      and it contains bad data
   */
  public void print(HttpServletResponse resp)
      throws IOException, SQLException, CaldataException
  {
    resp.setContentType("text/html; charset=iso-8859-1");
    ServletOutputStream out = resp.getOutputStream();
    getDocument().output(out);
  }

  /**
    Print the page to System.out (for testing)
    @exception IOException if there's a problem printing
    @exception SQLException if have to access the database, and
      there is a problem
    @exception CaldataException if we have to access the database,
      and it contains bad data
   */
  public void print() throws IOException, SQLException, CaldataException
  {
    getDocument().output(System.out);
  }

  /**
    Get the events for the user
    @return the events for the user
   */
  public EventsI getEvents()
  {
    return this.es;
  }

  /**
    Get the user of the application
    @return the user of the application
   */
  public User getUser()
  {
    return this.user;
  }

  /**
    Get the current session identifier 
    @return the current session identifier
   */
  public int getSessionIdentifier()
  {
    return this.user.getIdentifier();
  }

  /**
    Get the current day
    @return the current day
   */
  public MyCalendar getMc()
  {
    return this.mc;
  }
}
