// java

package uwcal.out.forms;

import org.apache.ecs.Element;
import org.apache.ecs.ElementContainer;
import org.apache.ecs.html.Form;
import org.apache.ecs.html.Input;
import org.apache.ecs.html.Span;
import org.apache.ecs.html.Table;
import org.apache.ecs.html.TD;
import org.apache.ecs.html.TR;

import edu.washington.cac.calendar.CalLogFactory;
import edu.washington.cac.calendar.CalParameter;
import edu.washington.cac.calendar.MyCalendar;

import uwcal.out.Page;
import uwcal.out.Style;
import uwcal.out.WebResources;

/** base class of most forms in the calendar
    @author Greg Barnes
    @version 1.0
  */
public abstract class CalendarForm 
{
  /** The page the form is a part of */
  protected Page page;

  /** Create a new instance
    @param page Page the form is a part of
   */
  CalendarForm(Page page) 
  {
    this.page = page;
  }

  /**
    Get the current date
    @return the current date
   */
  protected MyCalendar getMc()
  {
    return this.page.getMc();
  }

  protected abstract String formFunction();
  protected abstract String titleText();
  protected abstract Element[] inputElements();

  /**
    Get an array that says whether to add borders to the individual
    element tables
    @return an array that says whether to add borders to the individual
    element tables, null if there should be no such borders
   */
  protected boolean[] setElementBorders()
  {
    return null;
  }

  protected static Element hidden(String name, 
                                  int value) 
  {
      return new Input(Input.HIDDEN, name, value);
  }

  protected static Element hidden(String name, 
                              String value) 
  {
      return new Input(Input.HIDDEN, name, value);
  }

  /**
    Get the hidden parameters and value for the form
    @return the hidden parameters and value for the form
   */
  protected ElementContainer hiddens() 
  {
    return new ElementContainer().
        addElement(hidden(CalParameter.FUNCTION, formFunction())).
        addElement("\n").
        addElement(hidden(CalParameter.DATE, 
                          this.page.getMc().getDateDigits())).
        addElement("\n").
        addElement(hidden(CalParameter.FUNCTION2, this.page.nextFunction())).
        addElement("\n").
        addElement(hidden(CalParameter.IDENTIFIER_NAME, 
                          this.page.getSessionIdentifier())).
        addElement("\n");
  }

  private Element title() {
      return CommonFormElements.headerRow(titleText(), Style.FORM_TITLE);
  }

  /**
    Get the text shown in the submit button
    @return the text shown in the submit button
   */
  protected String submitButtonText() 
  {
    return FormsDisplayText.SUBMIT;
  }

  /**
    Get HTML for the submit button row
    @return HTML for the submit button row
   */
  protected Element submitButton() 
  {
    return CommonFormElements.submitButtonRow(submitButtonText());
  }

  /**
    Get the text explaining required fields
    @return the text explaining required fields
   */
  private Element requiredText() 
  {
    return new TR().
           addElement(new TD().
                      addElement(CommonFormElements.required()).
                      addElement(new Span().
                      addElement(FormsDisplayText.REQUIRED_INFO).
                      setClass(Style.FOOTNOTE)));
  }

  /**
    Get HTML for the end of the form
    @return HTML for the end of the form
   */
  protected Element end() 
  {
    return new ElementContainer().
           addElement(submitButton()).
           addElement("\n").
           addElement(requiredText()).
           addElement("\n");
  }

  /**
    Get a single form subcomponent, as HTML
    @param input The subcomponent
    @param addBorder whether to add a border around the subcomponent
    @return a single form subcomponent
   */
  private Element formInput(Element input, boolean addBorder) 
  {
    return 
        new ElementContainer().
            addElement(CommonFormElements.tableRowOneCell(
                new Table().addElement(input).setCellSpacing(0).
                setCellPadding(0).
                setBorder(addBorder ? 1 : 0).
setWidth("100%").       //%% for non-compliant browsers
                setClass(Style.FORM_ELEMENT))).
            addElement("\n");
  }

  /**
    Get the subcomponents of the form (usually inputs, but can be headers,
    spacers, text, etc.)
    @return the subcomponents of the form 
   */
  protected Element inputs() 
  {
    Element[] ie = inputElements();
    boolean[] addBorders = setElementBorders();
    ElementContainer ec = new ElementContainer();

    if (addBorders != null && addBorders.length != ie.length) {
      new CalLogFactory().create().
          println("CalendarForm::inputs: borders length " + addBorders.length +
                  " does not equal elements length " + ie.length);
      addBorders = null;
    }

    for (int i = 0; i < ie.length; i++) {
      if (addBorders == null) {
        ec.addElement(formInput(ie[i], false));
      } else {
        ec.addElement(formInput(ie[i], addBorders[i]));
      }
    }

    return ec;
  }

  public Form getForm() {
        return new Form(WebResources.BASE_POST_URL, Form.POST).
          addElement("\n").
          addElement(hiddens()).
          addElement(title()).addElement("\n").
          addElement(inputs()).
          addElement(end());
  }
}
